import fs from 'fs';
import path from 'path';
import dotenv from 'dotenv';

// 配置类型
export type Config = {
  port: number;
  tokenSecret?: string;
  redis?: {
    host?: string;
    port?: number;
    password?: string;
    version?: string;
    [key: string]: any;
  };
  posgtres?: {
    host?: string;
    port?: number;
    user?: string;
    password?: string;
    database?: string;
    [key: string]: any;
  };
  minio?: {
    endPoint?: string;
    bucketName?: string;
    useSSL?: boolean;
    accessKey?: string;
    secretKey?: string;
    [key: string]: any;
  };
  mongo?: {
    uri?: string;
    [key: string]: any;
  };
  [key: string]: any;
};
export const initConfig: Config = {
  port: 3000,
};
export const fileIsExist = (path: string) => {
  try {
    fs.accessSync(path, fs.constants.F_OK);
    return true;
  } catch (e) {
    return false;
  }
};
export const getDirname = () => {
  return process.cwd();
};
type ConfigOpts = {
  envConfigFile?: string;
  cwd?: string;
  fileName?: string;
};
/**
 * 查找文件，3级查找。往上查询
 * @param fileName 文件名
 * @returns
 */
export const getConfigFile = (opts?: ConfigOpts) => {
  if (opts?.envConfigFile) {
    const filePath = path.join(opts.cwd || getDirname(), opts.envConfigFile);
    if (fileIsExist(filePath)) {
      return filePath;
    }
  }
  const fileName = opts?.fileName || '.env';
  const dirname = opts?.cwd || getDirname();
  // 本级
  const benPath = dirname + '/' + fileName;
  const ben = fileIsExist(benPath);
  if (ben) return benPath;
  // 上级
  const lastPath = path.join(dirname, '../' + fileName);
  const last = fileIsExist(lastPath);
  if (last) return lastPath;
  // 上上级
  const lastLastPath = path.join(dirname, '../../' + fileName);
  const lastLast = fileIsExist(lastLastPath);
  if (lastLast) return lastLastPath;
  return '';
};

// 初始化读取配置文件
export const init = (opts?: ConfigOpts & { dotenvOpts?: dotenv.DotenvConfigOptions }): Config => {
  if (opts?.dotenvOpts) {
    const prased = dotenv.config(opts.dotenvOpts).parsed as Config;
    if (prased) {
      return prased;
    } else {
      throw new Error('未找到配置文件');
    }
  }
  // 配置读取路径，3级判断
  const filePath = getConfigFile(opts);
  console.log('config pathname:', filePath);
  if (!filePath) {
    throw new Error('未找到配置文件');
  }
  const value = dotenv.config({ path: filePath }).parsed as Config;
  return value;
};

/**
 * 从全局获取
 * @param initConfig 在全局未找到配置时，初始化配置的内容
 *
 * @returns Config
 */
export const useConfig = <T>(): Config & T => {
  const config = (global as any).config;
  const _config = config || init();
  !config && ((global as any)['config'] = _config);
  return _config;
};

export const useKey = (key: string, opts?: { defaultValue?: string; isNumber?: boolean; isBoolean?: boolean }): string | number | boolean => {
  let v = useConfig()[key];
  if (!v) {
    v = process.env[key];
  }
  if (!v) {
    return opts?.defaultValue || null;
  }
  if (opts?.isNumber) {
    return Number(v);
  }
  if (opts?.isBoolean) {
    return v === 'true';
  }
  return v;
};
